var amqp = require('amqplib/callback_api');
var mail = require('../mail/mail');
var router = require('../router/router');
var amqpConn = null;

function closeOnErr(err) {
  if (!err) return false;
  console.error("[AMQP] error", err);
  amqpConn.close();
  return true;
}

function work(msg, cb) {
    console.log("Got message from RMQ: ", msg.content.toString());
    router.handleReq(JSON.parse(msg.content.toString()));

    cb(true);
}

function processMsg(msg) {
  work(msg, function(ok) {
    try {
      if (ok)
        consChannel.ack(msg);
      else
        consChannel.reject(msg, true);
    } catch (e) {
      closeOnErr(e);
    }
  });
}

var consChannel = null;

function whenConnected() {
  amqpConn.createChannel(function(err, ch) {
    if (closeOnErr(err)) return;
    ch.on("error", function(err) {
      console.error("[AMQP] channel error", err.message);
    });
    ch.on("close", function() {
      console.log("[AMQP] channel closed");
    });

    consChannel = ch;

    ch.prefetch(10);
    ch.assertQueue("mailq", { durable: true }, function(err, _ok) {
      if (closeOnErr(err)) return;
      ch.consume("mailq", processMsg, { noAck: false });
      console.log("Worker is started");
    });
  });
}

function startRMQ() {
  amqp.connect(process.env.RMQ_ENDPOINT + "?heartbeat=60", function(err, conn) {
    if (err) {
      console.error("[AMQP]", err.message);
      return setTimeout(startRMQ, 1000);
    }
    conn.on("error", function(err) {
      if (err.message !== "Connection closing") {
        console.error("[AMQP] conn error", err.message);
      }
    });
    conn.on("close", function() {
      console.error("[AMQP] reconnecting");
      return setTimeout(startRMQ, 1000);
    });
    console.log("[AMQP] connected");
    amqpConn = conn;
    whenConnected();
  });
}

module.exports = { startRMQ };
